#ifndef FRAMEANIMATION_H
#define FRAMEANIMATION_H

#pragma once
#include "Animation.h"
#include "Timer.h"

//! Frame Animation class
/*!
	This class provides an templated interface for the frame animation.
*/
template<class TAnimation> class CFrameAnimation : public CAnimation
{

public:
	CFrameAnimation()
	{
		SetEntityType(Entity_FrameAnimation);

		m_uipDataIndex = NULL;
		m_pAnimationData = NULL;
		m_uiFrameCount = 
			m_uiLoopsCount = 
			m_uiCurrentLoop = 
			m_uiCurrentFrame = 
			m_uiDataCount = 0;
		m_bLoop = false;
		m_bAnimationComplete = false;
		m_bPaused = false;
		m_fLastTime = 0;
	};

	virtual ~CFrameAnimation()
	{
		Destroy();
	};
	/*!
		Method details:
		@param uiFrameCount - Number of frames for the animation
		@param uiDataCount - Number of items for the animation
		\return 1 if created succesfully, 0 otherwise
	*/
	virtual int Create(UINT uiFrameCount, UINT uiDataCount)
	{
		if(!(uiFrameCount > 0 && uiDataCount > 0))
			return 0;

		m_pAnimationData = new TAnimation*[uiFrameCount];
		m_uipDataIndex = new UINT[uiDataCount];

		for (UINT i=0; i<uiFrameCount; i++)
			m_pAnimationData[i] = NULL;

		for (UINT j=0; j<uiDataCount; j++)
			m_uipDataIndex[j] = 0;

		m_uiDataCount = uiDataCount;
		m_uiFrameCount = uiFrameCount;

		return 1;
	};

public:
	/*!
		Method details:
		@param uiFrameIndex - Frame index to return frame by
		\return Pointer to templated animation
	*/
	TAnimation *GetFrame(UINT uiFrameIndex)
	{
		if(uiFrameIndex >=0 && uiFrameIndex < m_uiDataCount)
			return m_pAnimationData[uiFrameIndex];

		return NULL;
	};
	/*!
		Method details:
		@param bLooping - Turn on looping for the animation
	*/
	void SetLooping(bool bLooping)
	{
		m_bLoop = bLooping;
	};
	/*!
		Method details:
		\return True if looping is enabled, false otherwise
	*/
	bool GetLooping()
	{
		return m_bLoop;
	};
	/*!
		Method details:
		@param uiLoopCount - Number of loops to repeat the animation
	*/
	void SetLoopCount(UINT uiLoopCount)
	{
		m_uiLoopsCount = uiLoopCount;
	};
	/*!
		Method details:
		\return Number of frames in the animation
	*/
	UINT GetFrameCount()
	{
		return m_uiFrameCount;
	};
	/*!
		Method details:
		\return Number of loops in the animation
	*/
	UINT GetLoopCount()
	{
		return m_uiLoopsCount;
	};
	/*!
		Method details:
		\return Current running loop index
	*/
	UINT GetLoop()
	{
		return m_uiCurrentLoop;
	};
	/*!
		Method details:
		\return Pointer to current frame
	*/
	TAnimation *GetCurrentFrame()
	{
		return GetFrame(m_uiCurrentFrame);
	};
	/*!
		Method details:
		\return Index of current frame
	*/
	UINT GetFrameIndex()
	{
		return m_uiCurrentFrame;
	};
	/*!
		Method details:
		@param fTime - time for current frame
	*/
	virtual void SetFrameTime(double fTime)
	{
		if(fTime > 0)
			m_fFrameTime = fTime;
	};
	/*!
		Method details:
		@param fTimePerFrame - Time to use per frame
		\return 1 if animation is still running, 0 otherwise
	*/
	virtual int Animate(double fTimePerFrame = -1)
	{
		if(m_bPaused == true)
			return 0;
		
		if(fTimePerFrame == -1 && CGlobalTimer::GetSingletonPtr() == NULL)
			return 0;
		else if(fTimePerFrame !=-1)
			m_fFrameTime = fTimePerFrame;

		double cur_time = CGlobalTimer::GetSingleton().GetAbsoluteTime();
		if(m_uiCurrentFrame != (m_uiFrameCount - 1))
		{
			if((cur_time - m_fLastTime) >= m_fFrameTime)
			{
				m_uiCurrentFrame++;
				m_fLastTime = cur_time;
			}
		}
		else
		{
			if(m_bLoop)
			{
				if(m_uiCurrentLoop != (m_uiLoopsCount - 1))
				{
					m_uiCurrentLoop++;
					m_bAnimationComplete = false;
				}
				else
					m_bAnimationComplete = true;
			}

			m_bAnimationComplete = true;
		}

		return (int)m_bAnimationComplete;
	};
	/*!
		Method details:
		@param fPauseTime - Pause for infinite amount of time or one specified by user
		\return 1 If animation is paused, 0 otherwise
	*/
	virtual int Pause(double fPauseTime = INFINITE)
	{
	
		return 1;
	};
	/*!
		Method details:
		@param fTimeToWait - Time to wait before resuming the animation
		\return 1 If it is still waiting, 0 otherwise
	*/
	virtual int Resume(double fTimeToWait = 0)
	{
	
		return 1;
	};
	/*!
		Method details:
		\return 1 If animation has been stopped, 0 otherwise
	*/
	virtual int Stop()
	{
	
		return 1;
	};

	public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename)
	{
		return CAnimation::LoadXML(pDataNode, strFilename);
	};
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename)
	{
		return CAnimation::SaveXML(pDataNode, strFilename);
	};

	virtual void Destroy()
	{
		SAFEDEL_ARRAY(m_pAnimationData)
		SAFEDEL_ARRAY(m_uipDataIndex)

		CAnimation::Destroy();
	};

	virtual bool IsOfType(eEntityType eType)
	{
		if(eType == Entity_FrameAnimation)
			return true;

		return CAnimation::IsOfType(eType);
	};

protected:

	UINT *m_uipDataIndex;			/*!< A list of data indices for each frame */
	UINT m_uiFrameCount;			/*!< Number of frames in this animation. */
	UINT m_uiLoopsCount;			/*!< Number of loops this animation has to run thru. */
	UINT m_uiCurrentLoop;			/*!< Current loop index. */
	UINT m_uiCurrentFrame;			/*!< Current frame value for the animation. */
	double m_fLastTime;				/*!< The last time the animation has been called successfully. */
	double m_fFrameTime;
	bool m_bLoop;					/*!< This variable stores the boolean value that determines if it has to loop thru the animation. */
	TAnimation	**m_pAnimationData;	/*!< These variables (n) store the data pointers used in each frame */
	UINT m_uiDataCount;				/*!< This variable stores a number of data entries from m_piDataIndices */
	bool m_bAnimationComplete;		/*!< This is a variable that indicates whether an animation has been completed. */
	bool m_bPaused;					/*!< The animation is paused when it is set to true */

	CTimer m_Timer;					/*!< A local timer */
};

#endif