#ifndef GUI_H
#define GUI_H

#include "TextureManager.h"
#include "MaterialManager.h"
#include "FontManager.h"
#include "AnimationManager.h"
#include "InputEngine.h"
#include "EventHandler.h"


#include <GL\gl.h>
#pragma comment (lib, "OpenGL32.lib")

class CGUIElement;

#define GUI_MAX_MESSAGES 64				/*!< Maximum messages for GUI Event Handler */
#define GUI_MAX_BUTTON_STATES 4			/*!< Default number of states for Button widget */
#define GUI_MINIMUM_WIDTH	8			/*!< Minimum width for ANY widget */
#define GUI_MINIMUM_HEIGHT	8			/*!< Minimum height for ANY widget */
#define DEFAULT_PROGRESSBAR_SPACINGX 3	/*!< Horizontal uiSpacing between Positive/Negative element' rectangles and Progress Bar rectangle */
#define DEFAULT_PROGRESSBAR_SPACINGY 3	/*!< Vertical uiSpacing between Positive/Negative element' rectangles and Progress Bar rectangle */
#define MENU_MAX_EXPANSIONS 8			/*!< Maximum expansions per EACH menu entry */
//#define GUI_USE_ACTIVE_INACTIVE 1		/*!< If this is defined, GUI will use Active/Inactive colors instead of materials */

#define GUI_DEFAULT_TITLEBAR_HEIGHT 10	/*!< Default Window Title Bar height */
#define GUI_DEFAULT_STATUSBAR_HEIGHT 8	/*!< Default Window Status Bar height */

#define GUI_TITLEBAR_BTN_SPACING 1		/*!< Horizontal/Vertical uiSpacing between Title Bar buttons and Title Bar rectangle */

enum eGUIFlag
{
	GUI_Flag_RestrictMovementX,		/*!< Flag: Restrict movement of the control in X Axis */
	GUI_Flag_RestrictMovementY,		/*!< Flag: Restrict movement of the control in Y Axis */
	GUI_Flag_RestrictSizeX,			/*!< Flag: Restrict width of the control */
	GUI_Flag_RestrictSizeY,			/*!< Flag: Restrict height of the control */
	GUI_Flag_BorderLock				/*!< Flag: Restrict using border to resize the widget */
};

enum eButtonState
{
	Button_Default = 0,				/*!< Button' default state (Mouse is out of the button and is not down) */
	Button_MouseOver,				/*!< Mouse is over Button (or Button-derived) widget */
	Button_MouseDown,				/*!< Mouse button is down on the Button (or Button-derived) widget */
	Button_Disabled					/*!< Button' disabled state (no mouse/keyboard messages are accepted) */
};

enum eWindowState
{
	GUI_WindowNormal,				/*!< Normal window state */
	GUI_WindowMinimized,			/*!< Window is minimized */
	GUI_WindowMaximized				/*!< Window is maximized */
};

enum eGUIControlType
{
	GUI_Element,					/*!< Control Type: Simple element (by default) */
	GUI_Border,						/*!< Control Type:  Border element */
	GUI_Static,						/*!< Control Type:  Static Control */
	GUI_TextBox,					/*!< Control Type:  TextBox Control */
	GUI_Button,						/*!< Control Type:  Button Control */
	GUI_RadioGroup,					/*!< Control Type:  RadioGroup Control */
	GUI_DropList,					/*!< Control Type:  Droplist Control */
	GUI_ListBox,					/*!< Control Type:  Listbox Control */
	GUI_HorizontalScrollBar,		/*!< Control Type:  Horizontal Scrollbar Control */
	GUI_VerticalScrollBar,			/*!< Control Type:  Vertical Scrollbar Control */
	GUI_HorizontalProgressBar,		/*!< Control Type:  Horizontal Progressbar Control */
	GUI_VerticalProgressBar,		/*!< Control Type:  Vertical Progressbar Control */
	GUI_WindowTitleBar,				/*!< Control Type:  Window Title Bar Element */
	GUI_WindowStatusBar,			/*!< Control Type:  Window Status Bar Element */
	GUI_Window,						/*!< Control Type:  Window Control */
	GUI_TabControl,					/*!< Control Type:  Tab Control */
	GUI_TabButton,					/*!< Control Type:  Tab Button (switch) */
	GUI_Menu,						/*!< Control Type:  Menu Control */
	GUI_MenuEntry,					/*!< Control Type:  Menu Sub-Entry Control */
	// Custom ones
	GUI_ProgressBar_PositiveElement,/*!< Control Type:  Progress Bar Positive element */
	GUI_ProgressBar_NegativeElement,/*!< Control Type:  Progress Bar Negative element */
	GUI_ScrollBar_ButtonInc,		/*!< Control Type:  Scroll Bar Increment button */
	GUI_ScrollBar_ButtonDec,		/*!< Control Type:  Scroll Bar Decrement button */
	GUI_ScrollBar_ButtonThumb,		/*!< Control Type:  Scroll Bar Thumb button */
	GUI_Window_TitleBar,			/*!< Control Type:  Window Title Bar Element */
	GUI_Window_StatusBar,			/*!< Control Type:  Window Status Bar Element */
	GUI_Window_MinimizeButton,		/*!< Control Type:  Window Minimize button */
	GUI_Window_MaxRestoreButton,	/*!< Control Type:  Window Maximize/Restore button */
	GUI_Window_CloseButton,			/*!< Control Type:  Window Close button */
	GUI_Caret,						/*!< Control Type:  Text Box Caret element */
	GUI,							/*!< Control Type:  GUI (Root) */
};

/*********** Messages for Control actions & outcomes ************/
enum eGUIMessage
{
	GUI_Message_Default = 250,		/*!< Message:  default message */
	GUI_Message_MouseOver,			/*!< Message:  Mouse is over the widget */
	GUI_Message_LMouseDown,			/*!< Message:  Left Mouse button is down (not pressed!) on the widget */
	GUI_Message_MouseOut,			/*!< Message:  Mouse leaves widget rectangle */
	GUI_Message_MouseMove,			/*!< Message:  General mouse movement */
	GUI_Message_RMouseDown,			/*!< Message:  Right Mouse button is down (not pressed!) on the widget */
	GUI_Message_LMouseUp,			/*!< Message:  Left Mouse button is released while still in the widget rectangle */
	GUI_Message_RMouseUp,			/*!< Message:  Right Mouse button is released while still in the widget rectangle */
	GUI_Message_KeyDown,			/*!< Message:  Keyboard key is down */
	GUI_Message_KeyUp,				/*!< Message:  Keyboard key is released */
	GUI_Message_KeyPress,			/*!< Message:  Keyboard key is pressed (down & up) */
	GUI_Message_AquireFocus,		/*!< Message:  Widget aquires focus (either via LMB Down/Up OR Mouse Over (or some other way) */
	GUI_Message_LostFocus,			/*!< Message:  Widget loses focus */
	GUI_Message_SizeX,				/*!< Message:  Widget changes it's width */
	GUI_Message_SizeY,				/*!< Message:  Widget changes it's height */
	GUI_Message_SizeXY,				/*!< Message:  Widget changes it's width AND height */
	GUI_Message_MoveX,				/*!< Message:  Widget has been moved on X Axis */
	GUI_Message_MoveY,				/*!< Message:  Widget has been moved on Y Axis */
	GUI_Message_MoveXY,				/*!< Message:  Widget has been moved on X AND Y Axii */
	GUI_Message_Visible,			/*!< Message:  Widget has changed its visibility to visible */
	GUI_Message_Hidden,				/*!< Message:  Widget has changed its visibility to hidden */
	// Custom ones
	GUI_Message_ButtonPressed,		/*!< Message:  Button (OR Button-derived control) has been PRESSED (Mouse Down then UP) */
	GUI_Message_Scrolled,			/*!< Message:  Scroll Bar (OR ScrollBar-derived control) has been scrolled (i.e changed it's current value) */
	GUI_Message_Progressed,			/*!< Message:  Progress Bar (OR ProgressBar-derived control) has changed it's current value */
	GUI_Message_TextChanged,		/*!< Message:  Text Box (OR TextBox-derived control) has changed it's text contents */
};

//! GUI Message structure
/*!
	This class represents a message for GUI, derived from generic tMessage structure.
	It contains	pointers to Sender and Receiver widgets in addition to standard message struct.
*/
struct tGUIMessage : public tMessage
{
	CGUIElement *pSender;	/*!< Who sent the message */
	CGUIElement *pReceiver;	/*!< Who receives the message */

	/*!< Constructor */
	tGUIMessage() : tMessage()
	{
		pSender = pReceiver = NULL;
	};

	/*!< Destructor */
	virtual ~tGUIMessage()
	{
	
	};
};

//! Button uiState structure
/*!
	This structure holds a list of events.\n\n

// There are N states for button\n
// Each state has a M events (e.g MOUSEOVER, MOUSEDOWN, KEYDOWN etc)\n

// Button wasn't pressed (m_pButtonState[0])\n
// m_pEventAnimations[0] - DISABLED State\n
// m_pEventAnimations[1] - NORMAL State\n
// m_pEventAnimations[2] - MOUSEOVER State\n
// m_pEventAnimations[3] - MOUSEDOWN State\n

// Button was pressed once (m_pButtonStates[1])\n
// m_pEventAnimations[0] - DISABLED State\n
// m_pEventAnimations[1] - NORMAL State\n
// m_pEventAnimations[2] - MOUSEOVER State\n
// m_pEventAnimations[3] - MOUSEDOWN State\n
*/

struct tButtonState
{
	CTextureFrameAnimation **m_pEventAnimations;	/*!< Reference to animation data (for each event) */
	int m_iEventCount;								/*!< Number of events */

	tButtonState() 
	{
		m_pEventAnimations = NULL;
		m_iEventCount = 0;
	};

	virtual ~tButtonState()
	{ SAFEDEL_ARRAY(m_pEventAnimations) };
};

//! GUI Event Handler
/*!
	This class represents a GUI Event Handler derived from generic Event Handler.
	It handles derived tGUIMessage structures for GUI messaging.
	@see tGUIMessage, CEventHandler
*/
class CGUIEventHandler : public CEventHandler
{
	CREATOR(CGUI);									/*!< Creator of this class */

protected:

	/*!< Constructor */
	CGUIEventHandler(UINT MaxMessages = GUI_MAX_MESSAGES);
	/*!< Destructor */
	virtual ~CGUIEventHandler();

	/*!
		Method details:
		@param uiMessageIndex Message index for the message
		\return Pointer to message structure if found, NULL if not
	*/
	tGUIMessage* GetMessage(UINT uiMessageIndex);

	/*!
		Method details:
		@param pSender Pointer to sender widget
		@param pReceiver Pointer to receiver widget
		@param eMessage Message to be sent
		@param lParam	Additional parameters (1)
		@param wParam	Additional parameters (2)
		@param uiTimeoutMs Timeout for the message (in milliseconds)
		\return 1 if sucessful, 0 if not
	*/
	virtual int PostMessage(CGUIElement *pSender, CGUIElement *pReceiver, eGUIMessage eMessage, DWORD lParam = 0, DWORD wParam = 0, UINT uiTimeoutMs = 0);
	/*!
		Method details:
		@param bOnce Dispatch messages only once and recycle them after
	*/
	virtual void Dispatch(bool bOnce = true);

	/*!
		Method details:
		Flushes all messages (e.g 'erases' them)
	*/
	void Flush();
	
};

//! Basic GUI Element
/*!
	This has properties of an XML Resource and is also a representation of a tree/node chain.
	It handles all basic inputs/outputs from various controls by default, or they can be overriden by derived classes.

*/
class CGUIElement :	public CXMLResource, public CDoubleList<CGUIElement *>
{
	CREATOR(CGUI);			/*!< Creator of this class */
	CREATOR(CGUIBorder);	/*!< Border has sub-contents of the element */

public:
	/*!< Constructor */
	CGUIElement();
	/*!< Destructor */
	virtual ~CGUIElement();

	/*!
		Method details:
		Initializes GUI Element' variables
	*/
	virtual void Initialize();
	
	/*!
		Method details:
		@param pParent - Pointer to parent widget
		@param WidgetRect - Rectangle for this widget
		@param pTexture - Pointer to texture for this widget
		@param pMaterial  -Pointer to material for this widget
		@param bBorder - Does this widget has border?
		\return 1 if sucessful, 0 if not
	*/
	virtual int Create(CGUIElement *pParent, tRect WidgetRect, CTexture *pTexture = NULL, CMaterial *pMaterial = NULL, bool bBorder = false);
	// Resource overrides
public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this element
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);

	/*!
		Method details:
		Draws the widget
	*/
	virtual void Draw();
	/*!
		Method details:
		Destroys the widget - e.g Pre-destructor (resets variables)
	*/
	virtual void Destroy();

public:
	/*!
		Method details:
		Processes messages from the GUI Event Handler
	*/
	virtual void ProcessMessages();
	/*!
		Method details:
		@param Message - Const. Reference to the message structure that is to be processed.
	*/
	virtual void ProcessMessage(tGUIMessage& Message);

	// Events
	/*!
		Method details:
		@param NewRect - New rectangle for the widget
		\return Varied returns @see GUI_Message_SizeX/Y/XY
	*/
	virtual int Resize(tRect& NewRect);

	/*!
		Method details:
		@param uiKeyCode - Key code
		\return Returns 1 if key has been processed (by widget OR by it's children), 0 if not
	*/
	virtual int OnKeyDown(UINT uiKeyCode);
	/*!
		Method details:
		@param uiKeyCode - Key code
		\return Returns 1 if key has been processed (by widget OR by it's children), 0 if not
	*/
	virtual int OnKeyUp(UINT uiKeyCode);
	/*!
		Method details:
		@param x - Units this widget has to be moved
		@param y - Units this widget has to be moved
		\return Varied return values @see GUI_Message_MoveX/Y/XY
	*/
	virtual int OnMove(UINT x, UINT y);
	/*!
		Method details:
		@param x - X Units the mosue has moved
		@param y - Y Units the mosue has moved
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnMouseMove(UINT x, UINT y);
	/*!
		Method details:
		@param x - X Location of the mouse when Left button has been held
		@param y - Y Location of the mouse when Left button has been held
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnLMouseDown(UINT x, UINT y);
	/*!
		Method details:
		@param x - X Location of the mouse when Left button has been released
		@param y - Y Location of the mouse when Left button has been released
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnLMouseUp(UINT x, UINT y);

	/*!
		Method details:
		@param x - X Location of the mouse when Right button has been held
		@param y - Y Location of the mouse when Right button has been held
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int OnRMouseDown(UINT x, UINT y);
	/*!
		Method details:
		@param x - X Location of the mouse when Right button has been released
		@param y - Y Location of the mouse when Right button has been released
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnRMouseUp(UINT x, UINT y);

	/*!
		Method details:
		\return Width of the widget
	*/
	UINT GetWidth();
	/*!
		Method details:
		@param uiWidth - new width of the widget
	*/
	void SetWidth(UINT uiWidth);

	/*!
		Method details:
		\return Height of the widget
	*/
	UINT GetHeight();
	/*!
		Method details:
		@param uiHeight - new height of the widget
	*/
	void SetHeight(UINT uiHeight);
	/*!
		Method details:
		@param uiIndex uiIndex of the child
		\return Pointer to the indexed child, NULL if nothing found
	*/
	CGUIElement *GetChild(UINT uiIndex);
	/*!
		Method details:
		@param eWidgetType - Type if the child
		\return Pointer to the child with type of (eWidgetType)
	*/
	CGUIElement *GetChild(eGUIControlType eWidgetType);
	/*!
		Method details: (DEPRICATED)
		@param pXMLElement - Pointer to the XML element that the child has
		\return Pointer to the child that hold pointer to the (pXMLElement)
	*/
	CGUIElement *GetChild(TiXmlElement *pXMLElement);

	/*!
		Method details:
		\return Child count for this widget
	*/
	UINT GetChildCount();
	/*!
		Method details:
		@param pChild - Pointer to the child to be removed
	*/
	void RemoveChild(CGUIElement *pChild);

	/*!
		Method details:
		@param iDeep - How deep to go and hide child widgets (-1 to hide all children, sub-children, etc)
	*/
	void HideSiblings(int iDeep = -1);
	/*!
		Method details:
		@param iDeep - How deep to go and show child widgets (-1 to show all children, sub-children, etc)
	*/
	void ShowSiblings(int iDeep = -1);

	/*!
		Method details:
		@todo Implement this function
	*/
	void SetZOrder(UINT uiZOrder);
	/*!
		Method details:
		@todo Implement this function
	*/
	UINT GetZOrder();

	/*!
		Method details:
		\return True if widget is visible, False otherwise
	*/
	bool Visible();

	/*!
		Method details:
		Hides this widget
	*/
	void Hide();
	/*!
		Method details:
		Makes this widget visible
	*/
	void Show();

	/*!
		Method details:
		\return Pointer to this widget' parent
	*/
	CGUIElement *GetParent();
	/*!
		Method details:
		@param pParent - Pointer to the parent for this widget
	*/
	void SetParent(CGUIElement *pParent);
	/*!
		Method details:
		@param pTexture - Pointer to the texture for this widget
	*/
	void SetTexture(CTexture *pTexture);
	/*!
		Method details:
		\return Pointer to this widget' texture
	*/
	CTexture *GetTexture();
	/*!
		Method details:
		@param pMaterial - Pointer to the material for this widget
	*/
	void SetMaterial(CMaterial *pMaterial);
	/*!
		Method details:
		\return Pointer to this widget' material
	*/
	CMaterial *GetMaterial();
	/*!
		Method details:
		@param uiIndex - Index for texture coordinate: \n 0 for Top Left, 1 - Top Right, 2 - Bottom Right, 3 - Bottom Left
		@param U, V - UV Coordinates for given corner (uiIndex)
	*/
	void SetTexCoord(UINT uiIndex, float U, float V);
	/*!
		Method details:
		\return Const. Reference to the specified corner'texture coordinates
	*/
	tVERTEX2f& GetTexCoord(UINT index);
	/*!
		Method details:
		\return Pointer to the Root GUI (Recursive!)
	*/
	CGUI *GetGUI();

	/*!
		Method details:
		@param NewRect - New rectangle for this widget
		This function assigns passed rectangle AND doesn't send messages (GUI_Message_SizeX/Y) and does not resize sub-children!
	*/
	void SetRect(tRect& NewRect);
	/*!
		Method details:
		\return Const. Reference to this widget' rectangle
	*/
	tRect& GetRect();

	/*!
		Method details:
		@param pGUI - Pointer to Root GUI
	*/
	void SetGUI(CGUI *pGUI);
	
	/*!
		Method details:
		\return Type of this widget (RTTI)
	*/
	eGUIControlType& GetType();
	/*!
		Method details:
		@param eWidgetType - Sets type of this widget (WARNING: Do not use in public!)
	*/
	void SetType(eGUIControlType eWidgetType);

	/*!
		Method details:
		@param eWidgetType - Sets type of this widget (WARNING: Do not use in public!)
		@param bSet - Sets the flag to the value of this parameter
	*/
	void SetFlag(eGUIFlag eFlag, bool bSet);
	/*!
		Method details:
		@param eFlag - returns True if this flag is set, False otherwise
	*/
	bool IsFlagSet(eGUIFlag eFlag);

	/*!
		Method details:
		@param pElement - returns True if the passed pointer is the child of this widget
	*/
	bool IsChild(CGUIElement *pElement);
	/*!
		Method details:
		\return True if Auto-Calculation is on, False otherwise
	*/
	bool	IsAutoCalc();
	/*!
		Method details:
		@param bAutoCalc - Sets Auto-Calculation to the value of this parameter
	*/
	void	SetAutoCalc(bool bAutoCalc);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

	CString GetElementType();
	
#ifdef GUI_USE_ACTIVE_INACTIVE
	bool IsActive() { return m_bActive; };
	void Activate() { m_bActive = true; };
	void Deactivate() { m_bActive = false; };
#endif

protected:
	
#ifndef			GUI_USE_ACTIVE_INACTIVE
	CMaterial	*m_pMaterial;			/*!< Pointer to widget' material */
#elif
	COLORREF	m_ActiveColor;			/*!< Material' Active Color (RGB) */
	COLORREF	m_InactiveColor;		/*!< Material' Inactive Color (RGB)*/
	bool		m_bActive;				/*!< Is this widget active or inactive*/
#endif
	//! Widget Core Attributes
	eGUIControlType m_eControlType;		/*!< Widget' Type */

	bool		m_bAutoCalc;			/*!< Auto-Calculation flag: used for various purposes */
	CGUIElement *m_pParent;				/*!< Pointer to the widget parent */
	CGUI		*m_pGUI;				/*!< Pointer to the Root GUI*/
	tRect		m_Rect;					/*!< Widget' Rectangle */
	DWORD		m_dwFlags;				/*!< Widget' flags */
	CTexture	*m_pTexture;			/*!< Pointer to widget' texture */
	tVERTEX2f	m_TexCoord[4];			/*!< Widget' texture coordinates */
	bool		m_bVisible;				/*!< Is widget visible */
};

//! Root GUI
/*!
	This class represents ROOT GUI Element that contains all of the controls.
	It handles basic commands, options & tuning and other various tasks.
*/
class CGUI : public CGUIElement
{
public:

	/*!< Constructor */
	CGUI();
	/*!< Destructor */
	virtual ~CGUI();
	/*!
		Method details:
		Initializes GUI' variables
	*/
	virtual void Initialize();
	/*!
		Method details:
		@param x - X Location of the mouse when Left button has been held
		@param y - Y Location of the mouse when Left button has been held
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnLMouseDown(UINT x, UINT y);
	/*!
		Method details:
		@param x - X Location of the mouse when Left button has been released
		@param y - Y Location of the mouse when Left button has been released
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int OnLMouseUp(UINT x, UINT y);
	/*!
		Method details:
		\return 1 if widget is created sucsessfully, 0 otherwise
	*/
	virtual int Create();
public:

	/*!
		Method details:
		Internal updates for GUI
	*/
	virtual void Update();

	/*!
		Method details:
		Draws the GUI
	*/
    virtual void Draw(); 

	/*!
		Method details:
		@param pElement - Pointer to element that is to be set as active (selected)
	*/
	void SetActiveElement(CGUIElement *pElement);
	/*!
		Method details:
		\return Pointer to element that is active (NULL if none)
	*/
	CGUIElement *GetActiveElement();

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this element
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);

	/*!
		Method details:
		@param pSender Pointer to sender widget
		@param pReceiver Pointer to receiver widget
		@param eMessage Message to be sent
		@param lParam	Additional parameters (1)
		@param wParam	Additional parameters (2)
		@param uiTimeoutMs Timeout for the message (in milliseconds)
		\return 1 if sucessful, 0 if not
	*/
	virtual int PostMessage(CGUIElement *pSender, CGUIElement *pReceiver, eGUIMessage eMessage, DWORD lParam = 0, DWORD wParam = 0, UINT uiTimeoutMs = 0);
	/*!
		Method details:
		Processes messages from the GUI Event Handler
	*/
	virtual void ProcessMessages();
	/*!
		Method details:
		@param Message Const. Reference to the message structure that is to be processed.
	*/
	virtual void ProcessMessage(tGUIMessage& Message);

	/*!
		Method details:
		@param uiMessageIndex Index to the message to be returned
		\return Pointer to message that is specified by (uiMessageIndex)
	*/
	tGUIMessage *GetMessage(UINT uiMessageIndex);
	/*!
		Method details:
		\return Number of messages
	*/
	UINT GetMessageCount();

	/*!
		Method details:
		\return True if mosue is locked, False otherwise
	*/
	bool IsLocked() { return m_bMouseLock; };
	/*!
		Method details:
		Locks the mouse input
	*/
	void Lock()		{ m_bMouseLock = true; };
	/*!
		Method details:
		Unlocks the mouse input
	*/
	void Unlock()	{ m_bMouseLock = false; };

	/*!
		Method details:
		Destroys the GUI (Pre-destructor)
	*/
	virtual void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:

	CGUIEventHandler m_EventHandler;	/*!< Root GUI Event Handler */
	CGUIElement *m_pActiveElement;		/*!< Current active element */
	bool		m_bMouseLock;			/*!< Is mouse locked? */
};

//! GUI Border
/*!
	This is a border class which consists of 8 elements (for now, it can have any amount of them).
	It tracks their movement and resizes parent widget accordingly.
	NOTE: can be locked.

*/
class CGUIBorder : public CGUIElement
{
public:
	CGUIBorder();
	virtual ~CGUIBorder();
	/*!
		Method details:
		Initializes GUI Border' variables
	*/
	virtual void Initialize();
	/*!
		Method details:
		@param pParent - Pointer to parent widget
		\return 1 if sucessful, 0 if not
	*/
	virtual int Create(CGUIElement *pParent);
	// Resource overrides
public:
	/*!
		Method details:
		Draws the widget
	*/
	virtual void Draw();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this element
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param NewRect - New rectangle for the widget
		\return Varied returns @see GUI_Message_SizeX/Y/XY
	*/
	virtual int Resize(tRect& NewRect);

	/*!
		Method details:
		@param x - X Location of the mouse when Left button has been held
		@param y - Y Location of the mouse when Left button has been held
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnLMouseDown(UINT x, UINT y);
	/*!
		Method details:
		@param x - X Location of the mouse when Left button has been released
		@param y - Y Location of the mouse when Left button has been released
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnLMouseUp(UINT x, UINT y);
	/*!
		Method details:
		@param x - X Units the mosue has moved
		@param y - Y Units the mosue has moved
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int OnMouseMove(UINT x, UINT y);

	/*!
		Method details:
		@param uiWidth - New size for the border
	*/
	void SetBorderWidth(UINT uiWidth);
	/*!
		Method details:
		\return Current border size
	*/
	UINT GetBorderWidth();

	/*!
		Method details:
		@param uiSpacing - New spacing between border' and element' rectangles
	*/
	void SetBorderSpacing(UINT uiSpacing);
	/*!
		Method details:
		\return Current border spacing 
	*/
	UINT GetBorderSpacing();
	
	/*!
		Method details:
		@param Message - Const. Reference to the message structure that is to be processed.
	*/
	virtual void ProcessMessage(tGUIMessage& Message);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

private:
	int				m_iBorderWidth,			/*!< Border Width */
					m_iBorderSpacing;		/*!< Border Spacing */
	UINT			m_uiDragElement;		/*!< Index of dragged element */
};

//! Static Control
/*!
	This is basically a CGUIElement with the ability to display text using provided Bitmap Font
	and additional parameters for it (alignment, etc)

*/
class CGUIStatic : public CGUIElement
{
public:

	CGUIStatic();
	virtual ~CGUIStatic();
	/*!
		Method details:
		Initializes Static Control' variables
	*/
	virtual void Initialize();
	/*!
		Method details:
		@param pParent - Pointer to parent widget
		@param WidgetRect - Rectangle for this widget
		@param pFont - Pointer to CCustomFont derived font for the widget to draw the text
		@param pTexture - Pointer to texture
		@param pMaterial - Pointer to material
		@param bBorder - does this widget has a border?
		\return 1 if sucessful, 0 if not
	*/
	virtual int Create(CGUIElement *pParent, tRect WidgetRect, CCustomFont *pFont = NULL, CTexture *pTexture = NULL, CMaterial *pMaterial = NULL, bool bBorder = false);
	// Resource overrides
public:
	/*!
		Method details:
		Draws the widget
	*/
	virtual void Draw();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this element
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param NewRect - New rectangle for the widget
		\return Varied returns @see GUI_Message_SizeX/Y/XY
	*/
	virtual int Resize(tRect& NewRect);
	/*!
		Method details:
		Destroys the widget - e.g Pre-destructor (resets variables)
	*/
	virtual void Destroy();

	/*!
		Method details:
		Draws only text that this Static (or Static-derived) Control has
	*/
	void DrawText();

	/*!
		Method details:
		\return Pointer to CCustomFont derived font
	*/
	const CCustomFont *GetFont();
	/*!
		Method details:
		@param pBitmapFont - Pointer to Bitmap Font (Only bitmap!)
	*/
	void SetFont(CBitmapFont *pBitmapFont);

	/*!
		Method details:
		\return Const. Reference to CString containing text
	*/
	CString& GetText();
	/*!
		Method details:
		@param strText - Const. Reference to new text for this widget
	*/
	void SetText(CString& strText);

	/*!
		Method details:
		@param strText - Const. Reference to new text for this widget
	*/
	void SetTextAlign(DWORD dwTextAlignFlags);
	/*!
		Method details:
		\return DWORD containing text alignment flags
	*/
	DWORD GetTextAlign();
	/*!
		Method details:
		\return Length of the entire text (including new lines)
	*/
	UINT GetTextLength();
	/*!
		Method details:
		\return Line count in the text
	*/
	UINT GetLineCount();
	/*!
		Method details:
		\return Line count in the text
	*/
	UINT GetLineLength(UINT uiLine);
	/*!
		Method details:
		@see m_bTextVisible
	*/
	void ShowText();
	/*!
		Method details:
		@see m_bTextVisible
	*/
	void HideText();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:

	// Attributes
	int		m_iHorizontalAlign;			/*!< Horizontal Text alignment (left, center or right) @see eTextAlign */
	int		m_iVerticalAlign;			/*!< Vertical Text alignment (top, middle or bottom) @see eTextAlign */

	/*!
		Method details:
		@param uiLine - Line index
		@param uiCharacterCount - Number of characters OF the line to return (0 for none)
		\return CString containing text for the specified line
	*/
	CString GetLine(UINT uiLine, UINT uiCharacterCount = 0);

	tVERTEX2d		m_TextPos;			/*!< XY Text coordinate (top left of first character) */
	CString			m_strText;			/*!< Text CString */
	CCustomFont		*m_pFont;			/*!< Pointer to font */
	bool			m_bTextVisible;		/*!< If True, text will be drawn, otherwise not */
};


//! Button Control
/*!
	This control has a possibility of being derived from CGUITextBox to be able to edit text,
	and at the same time maintain it's animations based on user mouse input.
	@see tButtonAnimation
*/
class CGUIButton : public CGUIStatic  
{

public:
	CGUIButton();
	virtual ~CGUIButton();
	/*!
		Method details:
		Initializes Button Control' variables
	*/
	virtual void Initialize();
	/*!
		Method details:
		@param pParent - Pointer to parent widget
		@param WidgetRect - Rectangle for this widget
		@param pFont - Pointer to CCustomFont derived font for the widget to draw the text
		@param pTexture - Pointer to texture
		@param pMaterial - Pointer to material
		@param bBorder - does this widget has a border?
		\return 1 if sucessful, 0 if not
	*/
	virtual int Create(CGUIElement *pParent, tRect WidgetRect, CCustomFont *pFont = NULL, CTexture *pTexture = NULL, CMaterial *pMaterial = NULL, bool bBorder = false);
	// Resource overrides
public:
	/*!
		Method details:
		Draws the widget
	*/
	virtual void Draw();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this element
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Destroys the widget - e.g Pre-destructor (resets variables)
	*/
	virtual void Destroy();
	/*!
		Method details:
		@param x - X Location of the mouse when Left button has been released
		@param y - Y Location of the mouse when Left button has been released
		\return 1 if it has been processed by widget (or it's children), 0 if not
	*/
	virtual int	OnLMouseUp(UINT x, UINT y);
	/*!
		Method details:
		\return Current state index 
		@see m_iCurrentState
	*/
	UINT GetState();
	//! Sets button state
	void SetState(bool bLoopAnimation, UINT uiState);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:
	/*!
		Method details:
		@param uiEventIndex Index for the event to fetch
		\return Pointer to the event specified by uiEventIndex value
	*/
	CTextureFrameAnimation *GetEventAnimation(UINT uiEventIndex);
	/*!
		Method details:
		\return Pointer to current animation frame (based off current mouse location/status)
	*/
	CTextureFrameAnimation *GetCurrentAnimation();
	/*!
		Method details:
		@param pButtonXMLNode Pointer to XML node containing data for Button State(s)
		\return True of all has been loaded succesfully, False otherwise
	*/
	bool LoadButtonStates(TiXmlNode *pButtonXMLNode);
	/*!
		Method details:
		@param pStateXMLNode Pointer to XML node containing data for State Events
		\return True of all has been loaded succesfully, False otherwise
	*/
	bool LoadStateEvents(tButtonState *uiState, TiXmlNode *pStateXMLNode);
	/*!
		Method details:
		@param pDataNode Pointer to Button XML Node
		\return 1 if all has been saved succesfully, 0 if not
	*/
	int SaveButtonStates(TiXmlNode *pDataNode);
	/*!
		Method details:
		@param pDataNode Pointer to Button State
		\return 1 if all has been saved succesfully, 0 if not
	*/
	int SaveStateEvents(TiXmlNode *pDataNode, tButtonState *pState);

private:
	tButtonState *m_pButtonStates;	/*!< Button states fixed-size array */
	UINT m_iuStateCount;			/*!< Number of button states */
	int m_iCurrentState;			/*!< Current state of the button */
	bool m_bLoopState;				/*!< Do we have to loop states? */
};


//! RadioGroup Control class
/*!
	This class is a collection of buttons that are part of\n
	this radio group.
*/
class CGUIRadioGroup : public CGUIStatic
{
	
public:
	CGUIRadioGroup();
	virtual ~CGUIRadioGroup();
	/*!
		Method details:
		Initializes GUI Border' variables
	*/
	virtual void Initialize();
	/*!
		Method details:
		@param pParent - Pointer to parent widget
		@param WidgetRect - Rectangle for this widget
		@param pFont - Pointer to CCustomFont derived font for the widget to draw the text
		@param pTexture - Pointer to texture
		@param pMaterial - Pointer to material
		@param bBorder - does this widget has a border?
		\return 1 if sucessful, 0 if not
	*/
	virtual int Create(CGUIElement *pParent, tRect WidgetRect, CCustomFont *pFont = NULL, CTexture *pTexture = NULL, CMaterial *pMaterial = NULL, bool bBorder = false);
	// Resource overrides
public:
	/*!
		Method details:
		Draws the widget
	*/
	virtual void Draw();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);

	/*!
		Method details:
		@param iSelection - Sets current radiogroup selection to the one specified by value (by index).
	*/
	virtual void SetCurSel(int iSelection);
	/*!
		Method details:
		\return Index of current selection
	*/
	int GetCurSel();
	/*!
		Method details:
		@param Message - Const. Reference to the message structure that is to be processed.
	*/
	virtual void ProcessMessage(tGUIMessage& Message);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:
	int m_iCurBtnSel;			/*!< Current selection index - '-1' if none is selected */
};

//! DropList Control
/*!
	This control represents a simple drop list. It is a derivative of CGUIButton, thus it can be clicked upon to expand the list.
	NOTE: All of its entries MUST be CGUIButton (or CGUIButton-derived) widgets.
*/
class CGUIDropList : public CGUIButton
{

public:
	CGUIDropList();
	virtual ~CGUIDropList();
	/*!
		Method details:
		Initializes GUI DropList' variables
	*/
	virtual void Initialize();
	/*!
		Method details:
		@param pParent - Pointer to parent widget
		@param WidgetRect - Rectangle for this widget
		@param pFont - Pointer to CCustomFont derived font for the widget to draw the text
		@param pTexture - Pointer to texture
		@param pMaterial - Pointer to material
		@param bBorder - does this widget has a border?
		@param pDefaultEntryBtn - Pointer to the default DropList entry CGUIButton (or -derived) widget
		\return 1 if sucessful, 0 if not
	*/
	virtual int Create(CGUIElement *pParent, tRect WidgetRect, CCustomFont *pFont = NULL, CTexture *pTexture = NULL, CMaterial *pMaterial = NULL, bool bBorder = false, CGUIButton *pDefaultEntryBtn = NULL);
	// Resource overrides
public:
	/*!
		Method details:
		Draws the widget
	*/
	virtual void Draw();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this element
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param NewRect - New rectangle for the widget
		\return Varied returns @see GUI_Message_SizeX/Y/XY
	*/
	virtual int Resize(tRect& NewRect);
	/*!
		Method details:
		@param Message - Const. Reference to the message structure that is to be processed.
	*/
	virtual void ProcessMessage(tGUIMessage& Message);

	/*!
		Method details:
		Expands the drop list
	*/
	virtual void Expand();
	/*!
		Method details:
		Collapses the drop list
	*/
	virtual void Collapse();

	
public:
	/*!
		Method details:
		@param iSelection - Sets current drop list selection to the one specified by value (by index).
	*/
	virtual void SetCurSel(int iSelection);
	/*!
		Method details:
		\return Index of current selection
	*/
	int GetCurSel();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:
	int		m_iSpacing,				/*!< Spacing between items (and drop list) */
			m_iListType;			/*!< List types:\n
											0: All entries are of the same width\n
											1: All entries are of the same width of maximum entry and droplist is of that max. width\n
											2: All entries are own width, but droplist becomes of max. width */
	bool	m_bExpanded,			/*!< Indicates when list is expanded */
			m_bIdentSize;			/*!< All entries are of the same size? */
	int m_iCurSel;					/*!< Currently selected item (-1 for none) */
	
};
/*!
	Method details:
	@param Rect - Rectangle to test the point against
	@param X, Y - point coordinates
	\return True if point is within given rectangle (inclusive boundaries), False otherwise
*/
bool PointInRect(tRect& Rect, int X, int Y);

#endif