#ifndef MATERIAL_H
#define MATERIAL_H

#pragma once
#include "XMLResource.h"
#include "Color.h"

//! Material class
/*!
	This class provides means for 4 different material colors
	as well as loading/saving it in XML.
*/
class CMaterial : public CXMLResource
{
protected:

	CREATOR(CMaterialManager);

	CMaterial();
	virtual ~CMaterial();

public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the material
		@param strFilename - String containing filename where the data for the material is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this material
		@param strFilename - String containing filename where the data for the material is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Binds this material as current one
	*/
	virtual void Bind() = 0;
	/*!
		Method details:
		\return Const. Reference to the Ambient color structure
	*/
	tRGBA& GetAmbient();
	/*!
		Method details:
		\return Const. Reference to the Diffuse color structure
	*/
	tRGBA& GetDiffuse();
	/*!
		Method details:
		\return Const. Reference to the Emissive color structure
	*/
	tRGBA& GetEmissive();
	/*!
		Method details:
		\return Const. Reference to the Specular color structure
	*/
	tRGBA& GetSpecular();
	/*!
		Method details:
		\return Const. Reference to the amount of shininess
	*/
	float& GetShininess();

	/*!
		Method details:
		@param ambient - RGBA for the ambient component
	*/
	void SetAmbient(tRGBA ambient);
	/*!
		Method details:
		@param diffuse - RGBA for the diffuse component
	*/
	void SetDiffuse(tRGBA diffuse);
	/*!
		Method details:
		@param emissive - RGBA for the emissive component
	*/
	void SetEmissive(tRGBA emissive);
	/*!
		Method details:
		@param specular - RGBA for the specular component
	*/
	void SetSpecular(tRGBA specular);
	/*!
		Method details:
		@param fShininess - amount of shininess
	*/
	void SetShininess(float fShininess);
	/*!
		Method details:
		Clean-up
	*/
	virtual void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:

	tRGBA			m_Ambient,		/*!< Ambient component */
					m_Diffuse,		/*!< Diffuse component */
					m_Emissive,		/*!< Emissive component */
					m_Specular;		/*!< Specular component */

	float			m_fShininess;	/*!< Shininess component */
};

#endif