#ifndef TEXTURE_H
#define TEXTURE_H

#include "XMLResource.h"
#include "FreeImage.h"
#include "Color.h"

#pragma once
#ifdef _DEBUG
	#pragma comment (lib, "FreeImageD.lib")
#endif

#ifdef _RELEASE
	#pragma comment (lib, "FreeImage.lib")
#endif

//! Texture class
/*!
	This class is a generic texture wrapper for FreeImage (FBITMAP),
	that can load a variety of image file formats.
*/
class CTexture : public CXMLResource
{
public:
    CTexture();
	virtual ~CTexture();

	virtual int LoadImage(CString strFilename);

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the texture
		@param strFilename - String containing filename where the data for the texture is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this texture
		@param strFilename - String containing filename where the data for the texture is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Binds this texture as current one
	*/
	virtual void Bind();
	/*!
		Method details:
		Clean-up
	*/
	void Destroy();
	/*!
		Method details:
		Releases the texture
	*/
	virtual void Release();
public:
	/*!
		Method details:
		\return Depth of the texture
	*/
	UINT GetDepth();
	/*!
		Method details:
		\return Width of the texture
	*/
	UINT GetWidth();
	/*!
		Method details:
		\return Height of the texture
	*/
	UINT GetHeight();
	/*!
		Method details:
		\return Pointer to FreeImage structure that stores the raw image data
	*/
	FIBITMAP *GetBitmap();
	/*!
		Method details:
		@param pPixel - Pointer to RGBA structure that will contain pixel RGBA values
		@param x,y - pixel coordinates
	*/
	void GetPixel(tRGBA *pPixel, UINT x, UINT y);
	/*!
		Method details:
		@param x,y - pixel coordinates
		@param color - new RGBA value for the pixel
	*/
	void SetPixel(UINT x, UINT y, tRGBA color);
	/*!
		Method details:
		\return Const. Reference to texture' filename
	*/
	CString& GetTextureFilename();
	/*!
		Method details:
		\return True if this texture has a mask, False otherwise
	*/
	bool IsMasked();
	/*!
		Method details:
		\return Const. Reference to RGBA value of the mask color
	*/
	tRGBA& GetMask();
	/*!
		Method details:
		@param color - New color for the mask
	*/
	void SetMask(tRGBA color);
	/*!
		Method details:
		\return 1 if created succesfully, 0 otherwise
	*/
	virtual UINT Create();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:

	tRGBA			m_Mask;						/*!< Mask color */
	FIBITMAP		*m_pBitmap;					/*!< Pointer to FreeImage structure for raw image data */
	CString			m_strTextureFilename;		/*!< Texture filename */
};

#endif

